/*
 * Driver for the Futaba TOSD-5711BB VFD display as fitted to the Antec Artisan
 * media centre PC. The black case with the 7 digit VFD. (VID:0547 PID:7000)
 *
 * Author: Steve Williams
 *
 * With thanks to Adam Williams for his help with the spelling
 *
 * This links against libusb V1.0 but the dummy module 
 * "futabaquirks" MUST be loaded first otherwise this does not work.
 *
 *                     M O D I F I C A T I O N    H I S T O R Y
 * Date												Mod
 * 29th March 2009				Work started
 *                   
 */
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <libusb-1.0/libusb.h>
#include <string.h>
#include <ctype.h>

#include "futaba.h"

//-----------------------------Defines------------------------------------------
#define SET_REPORT 0x09

#define VENDOR_ID 0x0547
#define PRODUCT_ID 0x7000

//-----------------------------Globals------------------------------------------
char *g_theString;
int g_theVolume;

// The one and only driver object
futabaDriver_t g_futabaDriver;

//--------------------------The implimentation---------------------------------
int futabaSendReport(libusb_device_handle *handle, futabaReport_t *report)
{
	int retVal = -1;
	uint8_t *p_rep = (uint8_t *)report;

	retVal  = libusb_control_transfer(
					handle,
					LIBUSB_DT_HID,				// Request Type
					SET_REPORT,					// Request
					0x0200,						// Report Type OUTPUT | ID 0
					0,							// Endpoint
					p_rep,						// Data to go
					sizeof(futabaReport_t),		// Length
					5000);

	return(retVal != sizeof(futabaReport_t));
}

int futabaSendString(libusb_device_handle *handle, char *string)
{
	int len;
	int n;
	futabaReport_t report;

	memset(&report,0,sizeof(futabaReport_t));

	len = strlen(string);
	if(len > FUTABA_STR_LEN)
	{
		len = FUTABA_STR_LEN;
		string[len] = 0;
	}

	report.opcode = FUTABA_OPCODE_STRING;
	report.param1 = 0x04;
	report.type.str.startPos = 0x01;
	report.type.str.len = len;

//	Display can only handle UPPER CASE
	for(n=0; n < len; n++)
	{
		report.type.str.string[n] = toupper(string[n]);
	}

	if( len > 7)
	{
		while( len >= 7)
		{
			if( futabaSendReport(handle, &report) )
			{	
				fprintf(stderr,"Failed in futabaSendString()\n");
				return -1;
			}
			report.type.str.startPos--;
			usleep(500000);
			len--;
		}
	}
	else
	{
		report.type.str.len = 7;
		return(futabaSendReport(handle,&report));
	}

	return 0;
}

int futabaSetVolume(libusb_device_handle *handle, int volPercent)
{
	int n;
	int numBars;
	futabaReport_t report;

	if( volPercent > 100 )
	{
		volPercent = 100;
	}

	if( volPercent < 0 )
	{
		volPercent = 0;
	}

	memset(&report,0,sizeof(futabaReport_t));

	numBars = ((volPercent*10*FUTABA_VOLUME_BARS)+50)/1000;

	report.opcode = FUTABA_OPCODE_SYMBOL;
	report.param1 = 0x02;
	report.type.sym.count = FUTABA_VOLUME_BARS;
	for(n=0; n < FUTABA_VOLUME_BARS; n++)
	{
		report.type.sym.symbol[n].symName = FUTABA_VOLUME_START+n;

		if( n <= numBars)
		{
			report.type.sym.symbol[n].state = FUTABA_SYM_ON;
		}
	}

	return(futabaSendReport(handle, &report));
}

int futabaInitDriver( futabaDriver_t *driver)
{
	int retVal = -1;

	if( driver->ctx == NULL )
	{
		if( libusb_init(&driver->ctx) < 0 )
		{
			fprintf(stderr,"usb_init() failed\n");
			retVal--;
		}
	}

//	Get a handle to our device
	if( (driver->handle = libusb_open_device_with_vid_pid(
				driver->ctx,
				VENDOR_ID,
				PRODUCT_ID)) == NULL )
	{
		fprintf(stderr,"libusb_open_Device_with_vid_pid() failed\n");
	}
	else
	{
		retVal++;
	}

	if( libusb_kernel_driver_active(driver->handle,0) )
	{
		printf("Kernel driver found. Will try do deactivate it....\n");

		if( libusb_detach_kernel_driver(driver->handle,0) )
		{
			fprintf(stderr,"Can't detach kernel driver\n");
			retVal--;
		}

		printf("Sucessfully detached the kernel driver\n");

	}

	return retVal;
}

int main(int argc, char **argv)
{
	int n;
	futabaDriver_t *driver = &g_futabaDriver;

	if( futabaInitDriver(driver) )
	{
		fprintf(stderr,"Failed to init driver\n");
		goto ENDIT;
	}

	if( argc >= 2 )
	{
		argv++;

		if( strcmp(*argv,"-r") == 0 )
		{
			printf("RESET\n");
			if( (n = libusb_reset_device(driver->handle)) )
			{
				fprintf(stderr,"Reset failed. Error %d\n",n);
				libusb_close(driver->handle);
				if( futabaInitDriver(driver) )
				{
					fprintf(stderr,"Failed to re-init driver\n");
					goto ENDIT;
				}
			}
		}
		else
		{
			if( argc == 2)
			{
				g_theString = *argv;
			}
		}

		if( strcmp(*argv,"-v") == 0 )
		{
			argv++;
			g_theVolume = atoi(*argv);
			printf("Setting volume to %d\n",g_theVolume);
		}	

	}

//	Claim the interface for us
	if( (n = libusb_claim_interface(driver->handle,0)) )
	{
		fprintf(stderr,"Failed to claim the interface. Error %d\n",n);
		libusb_close(driver->handle);
		libusb_exit(driver->ctx);
		exit(-1);
	}
	else
	{
		printf("We have our device\n");
	}

	if( g_theString == NULL )
	{
		g_theString = "MYTH TV";
	}

	if( futabaSendString(driver->handle,g_theString) )
	{
		fprintf(stderr,"futabaSendString() failed\n");
	}


	if( g_theVolume )
	{
		futabaSetVolume(driver->handle,g_theVolume);
	}

	futabaDaemon(driver->handle);

//	Clean up and exit
ENDIT:	

//	Reset
//	libusb_reset_device(handle);
	libusb_release_interface(driver->handle,0);
	libusb_close(driver->handle);
	libusb_exit(driver->ctx);
	return 0;
}

int futabaSleep(int seconds)
{
	usleep(seconds * 1000000);
	return seconds;
}

//	Gather the stats and display 'em
void futabaDaemon(libusb_device_handle *handle)
{
	int n;
	int secRollOver = 60;
	int secondsCount = 0;
	char timeStr[8];
	
	while(1)
	{
		n = getLoadAve();
		futabaSetVolume(handle,n);
		secondsCount += futabaSleep(5);

		if( secondsCount > secRollOver )
		{
			secRollOver = getTime(timeStr);
			futabaSendString(handle,timeStr);
			secondsCount = 0;
		}
	}
}

